#include "tvsserver_base.h"
#include "http_c_example_init.h"
#include <stdio.h>
#include <stdlib.h> 
//#include "http_c_example_handle.h"
//#include "http_c_example_bind.h"
#include "tvs_redefine_api_names.h"

char* supportedProtocols = "C_HTTP\0\0";

ProtocolSession *http_psess;
char* httpSupportedCommands[HTTP_AMMOUNT_OF_SUPPPORTED_COMMANDS] = {"GET", "POST", "HEAD"};
 
/*This function should return a list of protocols it supports*/
int __stdcall _GetProtocolNames(
	__out	char *buf,
	__inout	int *len
) {
    int size;
 
    if(len==0) return 0;

    size = GetNullTerminatedListSize(supportedProtocols); //calculate the size of the list

    //if the buffer is not big enough, return the size that is needed
    if(*len==0 || *len<size) {
        *len = size;
            return 1;
    }
 
    //copy the list to the buffer and return
	memcpy(buf, supportedProtocols, size);
    return 1;
}

/*This function initialize the protocols */
int __stdcall _StartProtocol(ProtocolSession *psess) {
    ControlPanelOptions *cpo;
    int retv=0;
    char* tempProtocolSignatureStructure=0;
 
    if(!strcmp(psess->protocolName, "C_HTTP")) { //init the HTTP protocol
        http_psess = psess;
 
        tempProtocolSignatureStructure = createProtocolSignatureFromArray(httpSupportedCommands, HTTP_AMMOUNT_OF_SUPPPORTED_COMMANDS);
 
        //Set the protocol type to PT_LABORER, make room for the MyHTTPstruct structure in the ListenSession and ClientSession and pass the supported API version.
		if(!SetProtocolInformation(
            psess, 
            PT_LABORER | PT_SUPPORT_PSIGN,
            (ProtocolSignature*) tempProtocolSignatureStructure,
            sizeof(MyHTTPstruct),
            V_TV_SERVER_1)
        )
            return false;
 
        //Create a ControlPanelOptions strcuture with room for 1 suboption.
        cpo = (ControlPanelOptions*) malloc(
            sizeof(ControlPanelOptions) + 
            sizeof(ControlPanelOption)*HTTP_AMMOUNT_OF_SUB_OPTIONS // +1 subOption
        );
 
        /*Empty memory and fill in the size, in bytes*/
        memset(cpo, 0, sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*HTTP_AMMOUNT_OF_SUB_OPTIONS);
        cpo->size = sizeof(ControlPanelOptions) + sizeof(ControlPanelOption)*HTTP_AMMOUNT_OF_SUB_OPTIONS;
 
        //The name of this protocol, communicated to the user, will be "HTTP-plugin"
        cpo->main.size = sizeof(ControlPanelOption);
        cpo->main.optionName  = "HTTP-C-plugin"; //the name of this protocol
 
        //suboption number 1
        cpo->subOptions[0].size = sizeof(ControlPanelOption);
        cpo->subOptions[0].name = "opt1"; //identifier name used by the function GetPluginData and SavePluginData
        cpo->subOptions[0].optionName = "Server-path"; //label of this option
        cpo->subOptions[0].userValue = (UserValueOption*) malloc(sizeof(UserValueOption));
        cpo->subOptions[0].userValue->type = TD_ASCII_STRING; //the pointer on defaultValue is a ascii string.
        cpo->subOptions[0].userValue->defaultValue = "c:\\WWWroot\\"; //default value of this option
        cpo->subOptions[0].userValue->defaultValueSize = strlen((char*) cpo->subOptions[0].userValue->defaultValue); 
        cpo->subOptions[0].flags = CPO_GEN_LISTENSESSION_OPTION; //a valid option for the listensession

        SetControlPanelOptions(psess, cpo);

        retv = AcceptProtocol(psess); //retv=false the protocol is rejected, retv=true the protocol is loaded
 
        free(cpo->subOptions[0].userValue);
        free(cpo);
    }
 
    //Free the ProtocolSignatureStructure if allocated
    if(tempProtocolSignatureStructure)
        free(tempProtocolSignatureStructure); 
 
    return retv;
}
 
char* createProtocolSignatureFromArray(char* p_array[], int ammountItems) {
	int bufSize, i, tmpLen;
	char* tmpBuf, *retBuf;

	//Calculate the size of every integer that will contain the size of the command (that last one is zero)
	bufSize = (ammountItems + 1) * sizeof(int);

	//Calculate the size of the buf members for the ProtocolSignature structure
	for(i=0; i<ammountItems; i++) {
		bufSize += strlen(p_array[i]);
	}

	//Allocate size for the ProtocolSignature structure
	retBuf = (char*) malloc(bufSize);

	tmpBuf = retBuf;
	//Copy every item with there size to the array
	for(i=0; i<ammountItems; i++) {
		tmpLen = strlen(p_array[i]);
		*(int*)tmpBuf = tmpLen;
		tmpBuf += sizeof(int);
		strcpy(tmpBuf, p_array[i]);
		tmpBuf += tmpLen;
	}

	*(int*)tmpBuf = 0;

	return retBuf;
}

/*This function will calculate the size, in bytes, of a null-terminated list*/
int GetNullTerminatedListSize(char* list) {
    int cnt=0;
    int len;
 
    if(list==0) return 0;
     
    while(*list!=0) { //The terminating null byte is found, this is the end of the list
        len = strlen(list);
        list += len + 1; //add the size of the item +1 to get the next item
        cnt += len + 1;
    }
    cnt++; //add the last zero byte
 
    return cnt; //return the size of the list
}