/*Initializes the TCP/IP connection for listening and initializes HTTP (however, this doesn't do annything yet)*/
#include <stdio.h>
#include <stdlib.h>
#include "tvsserver_base.h"
#include "tvs_redefine_api_names.h"
#include "c_example_init.h"
#include "c_example_bind.h"

int __stdcall _BindToProtocolCallBack(ProtocolSession *psess, ListenSession *lsess, int flags) {
    ProtocolData* pdata = FindProtocolDataStruct(psess, lsess->data);
    MyTCPIPstruct* tcpData;
    MyReplaystruct* replayData;
    struct sockaddr_in bindAddr={0};
    struct sockaddr_in6 bindAddr6={0};
	//The controlpanel always saves an byte/showt/integer as long long in 64-bit and as int in 32-bit
	#ifdef _WIN64
		long long listenPort;
		long long useVersion6;
    #else
		int listenPort;
		int useVersion6;
    #endif
    
    int len;
 
    if(psess==tcpip_psess) {
        tcpData = (MyTCPIPstruct*) &pdata->data;
 
        //Get the port this listenSession should listen to
		len = sizeof(listenPort);
        if(!GetPluginData(psess, lsess, (ClientSession*) 0, (char*) OPT_TCP_PORT, (char*) &listenPort, &len, GDPD_NAME_IS_INTEGER))
            return 0; //GetPluginData failed, return an error
 
		len = sizeof(useVersion6);
        if(!GetPluginData(psess, lsess, (ClientSession*) 0, (char*) OPT_TCP_VERSION6, (char*) &useVersion6, &len, GDPD_NAME_IS_INTEGER))
            return 0; //GetPluginData failed, return an error
 
        //Init a TCP/IP v4 of v6 socket
        if(useVersion6) {
            //return false if the socket failed to initialate
            tcpData->socket = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP);
            if(tcpData->socket==INVALID_SOCKET)
                return 0;
 
            //bind to the port to listen to
            bindAddr6.sin6_family = AF_INET6;
            bindAddr6.sin6_port = htons((int) listenPort);
            if(bind(tcpData->socket, (struct sockaddr*) &bindAddr6, sizeof(bindAddr6)))
                return 0;
        } else {
            //return false if the socket failed to initialate
            tcpData->socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
            if(tcpData->socket==INVALID_SOCKET)
                return 0;
 
            //bind to the port to listen to
            bindAddr.sin_family = AF_INET;
            bindAddr.sin_port = htons(listenPort);
            if(bind(tcpData->socket, (struct sockaddr*) &bindAddr, sizeof(bindAddr)))
                return 0;
        }
 
        //make the socket listen to the port
        if(listen(tcpData->socket, SOMAXCONN))
            return 0;

        //stop the socket if the other protocols in the lsess do not succees in initiating
        if(!RequestToBindToNextProtocol(psess, lsess)) {
            closesocket(tcpData->socket);
            return 0;
        }
 
        //return true, the function succeed
        return 1;
    //initiate the REPLAY protocol in the listen session
    } else if(psess==replay_psess) {
        replayData = (MyReplaystruct*) &pdata->data;
 
        //get the length of the server path
        len = 0;
        GetPluginData(psess, lsess, (ClientSession*) 0, "opt1", (char*) 0, &len, 0);
 
        //allocate memory to receive the memory path
        replayData->serverPath = (char*) malloc(len);


        //reveice the server path
        if(!GetPluginData(psess, lsess, (ClientSession*) 0, "opt1", replayData->serverPath, &len, 0))
            return 0; //GetPluginData failed, return an error
 
        //free the memory if the other Bind functions fail.
        if(!RequestToBindToNextProtocol(psess, lsess)) {
            free(replayData->serverPath);
            return 0;
        }
			
        return 1;
    }
	return 0;
}